#include "definition.h" 
#include "PinMapping.h"
#include "SAB.h"
#include "views.h"
#include "webApi.h"
#include "CFGDictionary.h"
#include "CFGParameterValue.h"

SAB sab;

unsigned long currentMs = 0, batteryMs = 0, buttonMs = 0;
View1Packet v1p = {sab.getRtcManager().getDateTime(), sab.getSdCardManager().getSize(GBYTE), sab.getPowerInfo(),0};
ViewAPPacket vap = {sab.getConnectivityManager().softAPmacAddress(), sab.getConnectivityManager().softAPSSID(), sab.getConnectivityManager().softAPIP(), sab.getConnectivityManager().softAPgetStationNum()};
ViewSTAPacket vstap = {sab.getConnectivityManager().macAddress(), sab.getConnectivityManager().localIP(), sab.getConnectivityManager().RSSI()};

void setup() 
{
  // put your setup code here, to run once:
  pinMode(GPIO_0, INPUT);
  
  Serial.println("Starting setup");
  CFGFileParser cfgFileParser(sab.getSdCardManager(), AP_CFG_FILE);
  CFGDictionary<CFGParameterValue> *cfgDictionary = (CFGDictionary<CFGParameterValue> *) cfgFileParser.parseFile();

  if(cfgDictionary != NULL)
  {
    Serial.print("AP PASSWORD : ");if((*cfgDictionary)("PASSWORD") != NULL)Serial.println((*cfgDictionary)("PASSWORD")->stringValue());
  }
  
  CFGFileParser cfgFileParsert1(sab.getSdCardManager(), "/CONFIG/TEST1.CFG");
  CFGFileParser cfgFileParsert2(sab.getSdCardManager(), "/CONFIG/TEST2.CFG");
  
  Serial.print("TEST1 : ");Serial.println(cfgFileParsert1.save(cfgDictionary));
  Serial.print("TEST2 : ");Serial.println(cfgFileParsert2.save(cfgDictionary));
  
  delete cfgDictionary;
  
  sab.getScreenManager().addView(&(view_1), &v1p, 0);
  sab.getScreenManager().addView(&(view_2), &vap, 1);
  sab.getScreenManager().addView(&(view_3), &vstap, 2);
  sab.getScreenManager().addView(&(memInfo), NULL, 3);
  sab.getScreenManager().addView(NULL, NULL, 4); //for testin purposes
  sab.getScreenManager().addView(&(dummy), NULL, 5); //for testin purposes
  
  sab.getScreenManager().displayView(0);
  if(sab.getRtcManager().hasLostPower())
  {
    Serial.println("Clock lost power");
    sab.getRtcManager().setDateTime(DateTime(F(__DATE__), F(__TIME__)));
  }
  
  sab.getWebServerManager().addApiRoutine("/sab/web/apitester", &(apiTesterApi), NULL);
  sab.getWebServerManager().addApiRoutine("/sab/view/next", &(nextViewApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/view", &(ViewByUIDApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/rtc/get/datetime", &(rtcGetTimeApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/rtc/set/datetime", &(rtcSetTimeApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/sdcard/size", &(sdCardSizeApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/sdcard/unmount", &(sdCardUnmountApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/sab/sdcard/mount", &(sdCardMountApi), &sab, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/esp/restart", &(espRestartApi), NULL, WEBServerManager::GET);
  sab.getWebServerManager().addApiRoutine("/esp/reset", &(espResetApi), NULL, WEBServerManager::GET);
  Serial.println("End setup");
}

void loop() 
{
  
  if(millis() - currentMs > 1000)
  {
    currentMs = millis();
    v1p.dateTime = sab.getRtcManager().getDateTime();
    v1p.nbViews = sab.getScreenManager().getViewCount();
    vap.ipAddr = sab.getConnectivityManager().softAPIP();
    vap.macAddr = sab.getConnectivityManager().softAPmacAddress();
    vap.nbOfCon = sab.getConnectivityManager().softAPgetStationNum();
    vstap.macAddr = sab.getConnectivityManager().macAddress();
    vstap.ipAddr = sab.getConnectivityManager().localIP();
    vstap.sigStrength = sab.getConnectivityManager().RSSI();
    
    sab.getScreenManager().displayView();
  }

  if(millis() - batteryMs > 10000)
  {
    batteryMs = millis();
    v1p.powerInfo = sab.getPowerInfo();
  }
    

  if(digitalRead(GPIO_0) == 0 && millis() - buttonMs > 500)
  {
    buttonMs = millis();
    Serial.println("Changing view");
    Serial.print("Selected view is : ");Serial.println(sab.getScreenManager().getCurrentViewUID());
    
    sab.getScreenManager().displayNextView();
  }

  //Run the webServer
  sab.getWebServerManager().runServer();
}
