#include "SAB.h"
#include "webApi.h"

boolean apiTesterApi(WEBServerManager::HttpRequestData &HRD, WiFiClient* wc, void* pData)
{
  wc->print(F("HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"API\" : \"available\" }"));
  return true;
}

boolean ViewByUIDApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  DictionaryHelper::StringEntity *pSE = HRD.getParams("UID");

  if(pSE == NULL)
  {
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"expected UID parameter\" }");
  }
  else if(strlen(pSE->getString()) > 0)
  {
    if(p->getScreenManager().displayView(atoi(pSE->getString())))
      sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"ViewUID\" : \"%d\" }", p->getScreenManager().getCurrentViewUID());
    else
      sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"%s\" }", p->getScreenManager().getErrorMessage());
  }
  else
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"UID parameter empty\" }");
  wc->print(buffer);
  return true;
}

boolean nextViewApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  if(p->getScreenManager().displayNextView())
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"ViewUID\" : \"%d\" }", p->getScreenManager().getCurrentViewUID());
  else
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"%s\" }", p->getScreenManager().getErrorMessage());
  wc->print(buffer);
  return true;
}

boolean rtcGetTimeApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  DateTime d = p->getRtcManager().getDateTime();
  sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"date\" : \"%d/%d/%d\", \"time\" : \"%d:%d:%d\" }", d.day(), d.month(), d.year(), d.hour(), d.minute(), d.second());
  wc->print(buffer);
  return true;
}

boolean rtcSetTimeApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  DictionaryHelper::StringEntity *pSE = HRD.getParams("datetime");
  if(pSE == NULL)
  {
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"expected datetime parameter\" }");
  }
  else if(strlen(pSE->getString()) > 0)
  {
    Dictionary<DictionaryHelper::StringEntity> *dictio = pSE->split('_');
    if(dictio == NULL)
    {
      sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"failed\", \"message\" : \"datetime format error\" }");
      wc->print(buffer);
      return true;
    }
    else
    {
      if(dictio->count() != 6)
      {
        sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"error\", \"message\" : \"datetime format error\" }"); 
      }
      else
      {
        p->getRtcManager().setDateTime(DateTime( 
          atoi(dictio->get(2)->getString()),
          atoi(dictio->get(1)->getString()),
          atoi(dictio->get((unsigned int)0)->getString()),
          atoi(dictio->get(3)->getString()),
          atoi(dictio->get(4)->getString()),
          atoi(dictio->get(5)->getString())
          ));
        DateTime d = p->getRtcManager().getDateTime();
        sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"date\" : \"%d/%d/%d\", \"time\" : \"%d:%d:%d\" }", d.day(), d.month(), d.year(), d.hour(), d.minute(), d.second());
      }
      
      wc->print(buffer);
      delete dictio;
      
      return true;
    }
  }
  wc->print(buffer);
  return true;
}

boolean sdCardUnmountApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  p->getSdCardManager().end();
  sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"card\" : \"unmounted\" }");
  wc->print(buffer);
  return true;
}

boolean sdCardMountApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  SAB *p = (SAB *)pData;
  char buffer[200];
  
  if(p->getSdCardManager().begin(p->getSdCardConfig().getSPISpeed(), p->getPinConfig().getSPI_sdCard_cs()))
    sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"card\" : \"mounted\" }");
  else
    sprintf(buffer,"HTTP/1.1 500 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"error\", \"message\" : \"begin failed\" }");
    
  wc->print(buffer);
  return true;
}

boolean espRestartApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  ESP.restart();

  return true;
}

boolean espResetApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
  ESP.restart();

  return true;
}

boolean sdCardSizeApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
	SAB *p = (SAB *)pData;
	char buffer[200];
	double size = p->getSdCardManager().getSize(GBYTE);
	if(size == 0.0)
		sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"card\" : \"not present\", \"size\" : \"0\" }");
	else
		sprintf(buffer,"HTTP/1.1 200 OK\r\nContent-Type: application/json\r\n\r\n{ \"status\" : \"ok\", \"card\" : \"present\", \"size\" : \"%.2f\", \"unit\" : \"GByte\" }",size);
	
	wc->print(buffer);
	return true;
}

boolean powerInfoApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
	SAB *p = (SAB *)pData;
	
	return true;
}

boolean wifiInfoApi(WEBServerManager::HttpRequestData &HRD, WiFiClient *wc, void *pData)
{
	SAB *p = (SAB *)pData;
	
	return true;
}

